/**   \file  suite_libdebug-memtrack.c
 *   \brief  Check Libdebug memtrack functions
 *  \author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  \internal
 *       Created:  24/06/2024
 *      Revision:  none
 * Last modified:  2024-07-28 16:36
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "debug/memtrack.h"
#include "oom.h"                                /* OOM simulation */
#include <check.h>
#include <unistd.h>                             /* pid_t getpid() */
#include <sys/stat.h>                           /* mkdir(), chdir(), mode_t */
#include <stdio.h>                              /* fprintf */
#include <stdlib.h>                             /* abort() */
#include "checktools.inc"

#ifndef NDEBUG
extern void memtrack_reset();
#endif

#ifndef NDEBUG
START_TEST(getallocatedRAM_first)
{
    unsigned int l_result;

    oomtest_fill(_i*sizeof(TMemBlock),2);
    l_result=memtrack_getallocatedRAM();
    oomtest_free();
    /* Always 0, except when initialized WithData */
    /* Uninitialized returns 0, Initialized returns 0, OOM uninitialized returns 0 */
    ck_assert(0==l_result);
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(getallocatedblocks_first)
{
    unsigned int l_result;
    oomtest_fill(_i*sizeof(TMemBlock),2);
    l_result = memtrack_getallocatedblocks();
    oomtest_free();
    /* Always 0, except when initialized WithData */
    /* Uninitialized returns 0, Initialized returns 0, OOM uninitialized returns 0 */
    ck_assert(l_result==0);
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(getblocksize_first)
{
    size_t l_result;

    oomtest_fill(_i*sizeof(TMemBlock),2);
    l_result = memtrack_getblocksize(NULL);
    oomtest_free();
    /* Always 0, because asking for "NULL" size */
    ck_assert(0==l_result);
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(getblocksize_null)
{
    uint64_t l_NbBlocks;
    size_t l_BlockSize;
    size_t l_RAMSize;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Run tests under constraint */
    /* Already initialized in previous lines, thus no need for allocation */
    oomtest_fill(_i,2);
    l_BlockSize=memtrack_getblocksize(NULL);
    oomtest_free();

    /* Check expected results */
    ck_assert_msg(0==l_BlockSize,"Blocksize of NULL pointer should be 0");

    /* Check no new block registered */
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(getblocksize_missing)
{
    uint64_t l_NbBlocks;
    size_t l_BlockSize;
    size_t l_RAMSize;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Already initialized in previous lines, thus no need for allocation */
    oomtest_fill(_i,2);
    l_BlockSize=memtrack_getblocksize((void*)999);
    oomtest_free();

    /* Check results */
    ck_assert_msg(0==l_BlockSize,"Blocksize of missing pointer should be 0");

    /* Check no new block registered */
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(addblock_null_first)
{
    unsigned int l_result;

    /* First test to trigger autoinit */
    /* Try to create/update a new blocks with null values */
    /* Initialize memtrack and fail */
    oomtest_fill(_i*sizeof(TMemBlock),2);
    l_result = memtrack_addblock(
                   NULL,       /* Pointer */
                   10,         /* Size */
                   __FILE__,   /* Source code filename */
                   __LINE__,   /* Source code line number */
                   __DATE__,   /* Compilation date */
                   __TIME__,   /* Compilation time */
                   __func__    /* Source code function, GNU extension */
               );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==l_result,"Should not be able to register a NULL pointer");
    ck_assert_msg(0==memtrack_getblocksize(NULL),"Blocksize of NULL pointer should be 0");
    /* Can not check the block counters value, it depends on WithData */
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif

#ifndef NDEBUG
START_TEST(delblock_null_first)
{
    unsigned int l_result;

    /* Initialize memtrack and fail */
    oomtest_fill(_i*sizeof(TMemBlock),2);
    l_result = memtrack_delblock(
                   NULL,       /* Pointer */
                   __FILE__,   /* Source code filename */
                   __LINE__,   /* Source code line number */
                   __DATE__,   /* Compilation date */
                   __TIME__,   /* Compilation time */
                   __func__    /* Source code function, GNU extension */
               );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==l_result,"Should not be able to unregister a NULL pointer");
    ck_assert_msg(0==memtrack_getblocksize(NULL),"Blocksize of NULL pointer should be 0");
    /* Can not check the block counters value, it depends on WithData */
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif

#ifndef NDEBUG
START_TEST(addblock_null)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Already initialized in previous lines, thus no need for allocation */
    /* Try to create/update a new blocks with null values, fails, no need for RAM */
    oomtest_fill(_i,2);
    result=memtrack_addblock(
               NULL,       /* Pointer */
               10,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==result,"Should not be able to register a NULL pointer");
    ck_assert_msg(0==memtrack_getblocksize(NULL),"Blocksize of NULL pointer should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif

#ifndef NDEBUG
START_TEST(addblock_duplicate)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Already initialized in previous lines, thus no need for allocation */
    /* Try to create/update one new block  */
    oomtest_fill(_i*sizeof(TMemBlock),2);
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               10,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();
    ck_assert((0==result)||(oomtest_enabled()));
    if (0!=result) {
        /* OOM occured, fix that */
        result=memtrack_addblock(
                   (void*)999, /* Pointer */
                   10,         /* Size */
                   __FILE__,   /* Source code filename */
                   __LINE__,   /* Source code line number */
                   __DATE__,   /* Compilation date */
                   __TIME__,   /* Compilation time */
                   __func__    /* Source code function, GNU extension */
               );
    }
    l_NbBlocks+=1;
    l_RAMSize+=10;

    /* Check results */
    ck_assert_msg(0==result,"Should able to register a pointer");
    ck_assert_msg(10==memtrack_getblocksize((void*)999),"Blocksize of pointer should be 10");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Add a duplicate, should fail */
    oomtest_fill(_i*sizeof(TMemBlock),2);
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               20,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==result,"Should not be able to register a duplicate pointer");
    ck_assert_msg(10==memtrack_getblocksize((void*)999),"Duplicate pointer updated the size");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Cleanup */
    result = memtrack_delblock(
                 (void*)999, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    ck_assert(0==result);
}
END_TEST
#endif

#ifndef NDEBUG
START_TEST(addblock_empty)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    /* Already initialized and empty val will fail, thus no need for RAM */
    oomtest_fill(_i,2);
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               0,          /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==result,"Should not be able to register a 0 sized pointer");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of missing block should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i,2);
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               1,          /* Size */
               "",         /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==result,"Should not be able to register a pointer without filename");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of missing block should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i,2);
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               1,          /* Size */
               __FILE__,   /* Source code filename */
               0,          /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==result,"Should not be able to register a pointer à line number 0");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of missing block should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i,2);
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               1,          /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               "",         /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==result,"Should not be able to register a pointer without a compilation date");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of missing block should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i,2);
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               1,          /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               "",         /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==result,"Should not be able to register a pointer without a compilation time");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of missing block should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i,2);
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               1,          /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               ""          /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert_msg(1==result,"Should not be able to register a pointer without a function name");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of missing block should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(addblock_longstrings)
{
    uint64_t l_NbBlocks;
    size_t l_BlockSize;
    size_t l_RAMSize;
    unsigned int result;
    char *szLongString =
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        ;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i*(sizeof(TMemBlock)+sizeof(*szLongString)),2);
    result=memtrack_addblock(
               (void*)999,  /* Pointer */
               1,           /* Size */
               szLongString,/* Source code filename */
               __LINE__,    /* Source code line number */
               __DATE__,    /* Compilation date */
               __TIME__,    /* Compilation time */
               __func__     /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        l_NbBlocks+=1;
        l_RAMSize+=1;
        l_BlockSize=1;
    } else {
        /* result is false while oom testing : OOM occured */
        l_BlockSize = 0;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(l_BlockSize==memtrack_getblocksize((void*)999),"Blocksize inconsistency");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i*(sizeof(TMemBlock)+sizeof(*szLongString)),2);
    result=memtrack_addblock(
               (void*)998,  /* Pointer */
               2,           /* Size */
               __FILE__,    /* Source code filename */
               1,           /* Source code line number */
               szLongString,/* Compilation date */
               __TIME__,    /* Compilation time */
               __func__     /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        l_NbBlocks+=1;
        l_RAMSize+=2;
        l_BlockSize=2;
    } else {
        /* result is false while oom testing : OOM occured */
        l_BlockSize = 0;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(l_BlockSize==memtrack_getblocksize((void*)998),"Blocksize inconsistency");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i*(sizeof(TMemBlock)+sizeof(*szLongString)),2);
    result=memtrack_addblock(
               (void*)997,  /* Pointer */
               3,           /* Size */
               __FILE__,    /* Source code filename */
               1,           /* Source code line number */
               __DATE__,    /* Compilation date */
               szLongString,/* Compilation time */
               __func__     /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        l_NbBlocks+=1;
        l_RAMSize+=3;
        l_BlockSize=3;
    } else {
        /* result is false while oom testing : OOM occured */
        l_BlockSize = 0;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(l_BlockSize==memtrack_getblocksize((void*)997),"Blocksize inconsistency");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to create/add a new  block with empty values */
    oomtest_fill(_i*(sizeof(TMemBlock)+sizeof(*szLongString)),2);
    result=memtrack_addblock(
               (void*)996,  /* Pointer */
               4,           /* Size */
               __FILE__,    /* Source code filename */
               1,           /* Source code line number */
               __DATE__,    /* Compilation date */
               __TIME__,    /* Compilation time */
               szLongString /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        l_NbBlocks+=1;
        l_RAMSize+=4;
        l_BlockSize=4;
    } else {
        /* result is false while oom testing : OOM occured */
        l_BlockSize = 0;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(l_BlockSize==memtrack_getblocksize((void*)996),"Blocksize inconsistency");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(delblock_null)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* Functor autoinit already tested in a previous test, can surround with sanity checks */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks count differs");

    /* Run tests under constraint */
    oomtest_fill(_i,2);
    result=memtrack_delblock(
               NULL,       /* Pointer */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results should always fail, oom or not */
    ck_assert_msg(1==result,"Should not be able to unregister a NULL pointer");
    ck_assert_msg(0==memtrack_getblocksize(NULL),"Blocksize of NULL pointer should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(delblock_missing)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* Functor autoinit already tested in a previous test, can surround with sanity checks */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Run tests under constraint */
    oomtest_fill(_i,2);
    result=memtrack_delblock(
               (void*)999, /* Pointer */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Check results should always fail, oom or not */
    ck_assert_msg(1==result,"Should not be able to unregister a missing pointer");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of missing pointer should be 0");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count changed");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM changed");
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(addblock1_delblock1)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"The 999 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               10,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=10;
        ck_assert_msg(10==memtrack_getblocksize((void*)999),"Blocksize of 999 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of 999 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)999, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=10;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of deleted 999 block should be 0");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(addblock12_delblock12)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"The 999 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               10,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=10;
        ck_assert_msg(10==memtrack_getblocksize((void*)999),"Blocksize of 999 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of 999 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)998),"The 998 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)998, /* Pointer */
               20,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=20;
        ck_assert_msg(20==memtrack_getblocksize((void*)998),"Blocksize of 998 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)998),"Blocksize of 998 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)999, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=10;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of deleted 999 block should be 0");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)998, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=20;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)998),"Blocksize of deleted 998 block should be 0");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(addblock12_delblock21)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"The 999 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               10,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=10;
        ck_assert_msg(10==memtrack_getblocksize((void*)999),"Blocksize of 999 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of 999 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)998),"The 998 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)998, /* Pointer */
               20,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=20;
        ck_assert_msg(20==memtrack_getblocksize((void*)998),"Blocksize of 998 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)998),"Blocksize of 998 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)998, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=20;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)998),"Blocksize of deleted 998 block should be 0");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)999, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=10;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of deleted 999 block should be 0");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(addblock123_delblock123)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"The 999 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               10,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=10;
        ck_assert_msg(10==memtrack_getblocksize((void*)999),"Blocksize of 999 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of 999 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)998),"The 998 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)998, /* Pointer */
               20,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=20;
        ck_assert_msg(20==memtrack_getblocksize((void*)998),"Blocksize of 998 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)998),"Blocksize of 998 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)997),"The 997 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)997, /* Pointer */
               30,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=30;
        ck_assert_msg(30==memtrack_getblocksize((void*)997),"Blocksize of 997 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)997),"Blocksize of 997 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)999, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=10;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of deleted 999 block should be 0");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)998, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=20;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)998),"Blocksize of deleted 998 block should be 0");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)997, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=30;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)997),"Blocksize of deleted 997 block should be 0");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(addblock123_delblock2)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    unsigned int result;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)999),"The 999 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)999, /* Pointer */
               10,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=10;
        ck_assert_msg(10==memtrack_getblocksize((void*)999),"Blocksize of 999 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)999),"Blocksize of 999 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)998),"The 998 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)998, /* Pointer */
               20,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=20;
        ck_assert_msg(20==memtrack_getblocksize((void*)998),"Blocksize of 998 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)998),"Blocksize of 998 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    /* Ensure that block does not already exist */
    ck_assert_msg(0==memtrack_getblocksize((void*)997),"The 997 block should not already exist");

    oomtest_fill(_i*sizeof(TMemBlock),2);
    /* Create a new block */
    result=memtrack_addblock(
               (void*)997, /* Pointer */
               30,         /* Size */
               __FILE__,   /* Source code filename */
               __LINE__,   /* Source code line number */
               __DATE__,   /* Compilation date */
               __TIME__,   /* Compilation time */
               __func__    /* Source code function, GNU extension */
           );
    oomtest_free();

    /* Intermediate check */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks+=1;
        l_RAMSize+=30;
        ck_assert_msg(30==memtrack_getblocksize((void*)997),"Blocksize of 997 block should be 10");
    } else {
        ck_assert_msg(0==memtrack_getblocksize((void*)997),"Blocksize of 997 missing block should be 0");
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    oomtest_fill(_i,2);
    /* Delete the created block */
    result = memtrack_delblock(
                 (void*)998, /* Pointer */
                 __FILE__,   /* Source code filename */
                 __LINE__,   /* Source code line number */
                 __DATE__,   /* Compilation date */
                 __TIME__,   /* Compilation time */
                 __func__    /* Source code function, GNU extension */
             );
    oomtest_free();

    /* Check results */
    ck_assert((0==result)||(oomtest_enabled()));
    if (0==result) {
        /* No OOM error, update the check counters */
        l_NbBlocks-=1;
        l_RAMSize-=20;
    }
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
    ck_assert_msg(0==memtrack_getblocksize((void*)998),"Blocksize of deleted 998 block should be 0");
}
END_TEST
#endif /* NDEBUG */

#ifndef NDEBUG
START_TEST(adddelblock_oom)
{
    uint64_t l_NbBlocks;
    size_t l_RAMSize;
    long unsigned int i=0;
    uint16_t oomtestvalues[65536] = {0};
    char *szLongString =
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        ;

    /* LibMemTrack consistency check preparation */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks mismatch");

    /* Try to trigger OOM inside the tested feature */
    i = 0;
    oomtest_fill(_i*(sizeof(TMemBlock)+4*(26+10+26)*10),2);
    do {
        i++;
        oomtestvalues[i] = memtrack_addblock(
                               (void*)(i+99),/* Pointer */
                               10,           /* Size */
                               szLongString, /* Source code filename */
                               __LINE__,     /* Source code line number */
                               szLongString, /* Compilation date */
                               szLongString, /* Compilation time */
                               szLongString  /* Source code function, GNU extension */
                           );
    } while ((i<65535)&&(0==oomtestvalues[i]));
    oomtest_free();

    /* Intermediate check */
    l_NbBlocks+=i-1;
    l_RAMSize+=(i-1)*10;
    /* Should have returned NULL (OOM) before reaching 65535 */
    ck_assert_msg(65535!=i,"Did not manage to trigger OOM");
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");

    /* We will loose i value, precalculate values */
    l_NbBlocks-=i-1;
    l_RAMSize-=(i-1)*10;

    oomtest_fill(_i,2);
    /* Free all the allocated memory */
    i = 1;
    while ((i<65535)&&(0==oomtestvalues[i])) {
        if(0!=(oomtestvalues[i]=memtrack_delblock(
                                    (void*)(i+99),/* Pointer */
                                    __FILE__,     /* Source code filename */
                                    __LINE__,     /* Source code line number */
                                    __DATE__,     /* Compilation date */
                                    __TIME__,     /* Compilation time */
                                    __func__      /* Source code function, GNU extension */
                                )))
            break;
        i++;
    }
    oomtest_free();

    ck_assert_msg(65535!=i,"Did not manage to free all allocated RAM");
    ck_assert(1==oomtestvalues[i]);
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"Block count inconsistency");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"Allocated RAM inconsistency");
}
END_TEST
#endif /* NDEBUG */

/************************************************************************/
void memtrack_checked_uninitialized_setup()
{
    signals_catch();
    forktest_gprofdir();
}
void memtrack_checked_withdata_setup()
{
    signals_catch();
    forktest_gprofdir();
#ifndef NDEBUG
    uint64_t l_NbBlocks;
    uint64_t l_RAMSize;

    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"getallocatedblocks does not match counter");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"getallocatedRAM does not match the counter");
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks count differs");
    ck_assert_msg(0==memtrack_addblock(
                      (void*)1,   /* Pointer */
                      10,         /* Size */
                      __FILE__,   /* Source code filename */
                      __LINE__,   /* Source code line number */
                      __DATE__,   /* Compilation date */
                      __TIME__,   /* Compilation time */
                      "fonction"  /* Source code function, GNU extension */
                  ),"Creating block (Normal values)");
    ck_assert_msg(10==memtrack_getblocksize((void*)1),"Invalid getblocksize");
    ck_assert_msg( 0==memtrack_getblocksize((void*)2),"Invalid getblocksize");
    ck_assert_msg( 0==memtrack_getblocksize((void*)3),"Invalid getblocksize");
    l_NbBlocks += 1;
    l_RAMSize += 10;
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"getallocatedblocks does not match counter");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"getallocatedRAM does not match the counter");
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks count differs");
    ck_assert_msg(0==memtrack_addblock(
                      (void*)2,   /* Pointer */
                      20,         /* Size */
                      __FILE__,   /* Source code filename */
                      __LINE__,   /* Source code line number */
                      __DATE__,   /* Compilation date */
                      __TIME__,   /* Compilation time */
                      "fonction"  /* Source code function, GNU extension */
                  ),"Creating block (Normal values)");
    ck_assert_msg(10==memtrack_getblocksize((void*)1),"Invalid getblocksize");
    ck_assert_msg(20==memtrack_getblocksize((void*)2),"Invalid getblocksize");
    ck_assert_msg( 0==memtrack_getblocksize((void*)3),"Invalid getblocksize");
    l_NbBlocks += 1;
    l_RAMSize += 20;
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"getallocatedblocks does not match counter");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"getallocatedRAM does not match the counter");
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks count differs");
    ck_assert_msg(0==memtrack_addblock(
                      (void*)3,   /* Pointer */
                      30,         /* Size */
                      __FILE__,   /* Source code filename */
                      __LINE__,   /* Source code line number */
                      __DATE__,   /* Compilation date */
                      __TIME__,   /* Compilation time */
                      "fonction"  /* Source code function, GNU extension */
                  ),"Creating block (Normal values)");
    ck_assert_msg(10==memtrack_getblocksize((void*)1),"Invalid getblocksize");
    ck_assert_msg(20==memtrack_getblocksize((void*)2),"Invalid getblocksize");
    ck_assert_msg(30==memtrack_getblocksize((void*)3),"Invalid getblocksize");
    l_NbBlocks += 1;
    l_RAMSize += 30;
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"getallocatedblocks does not match counter");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"getallocatedRAM does not match the counter");
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks count differs");
#endif /* NDEBUG */
}
void memtrack_checked_common_teardown()
{
    signals_release();
#ifndef NDEBUG
    uint64_t l_NbBlocks;
    uint64_t l_RAMSize;
    /** @todo make memtrack_reset compatible with OOMTEST_* helpers */
    l_NbBlocks=memtrack_getallocatedblocks();
    l_RAMSize=memtrack_getallocatedRAM();
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"getallocatedblocks does not match counter");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"getallocatedRAM does not match the counter");
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks count differs");
    memtrack_reset();
    memtrack_reset(); /* Test the double run protection */
    l_NbBlocks = 0;
    l_RAMSize = 0;
    ck_assert_msg(l_NbBlocks==memtrack_getallocatedblocks(),"getallocatedblocks does not match counter");
    ck_assert_msg(l_RAMSize==memtrack_getallocatedRAM(),"getallocatedRAM does not match the counter");
    ck_assert_msg(memtrack_getallocatedblocks()==memtrack_dumpblocks(),"Internal NbBlocks counter and dumpblocks count differs");
#endif /* NDEBUG */
}
/************************************************************************/
void memtrack_unchecked_common_setup()
{
    forktest_init();
}
void memtrack_unchecked_common_teardown()
{
}
void memtrack_unchecked_oom_setup()
{
    oomtest_enable(RAMLIMIT_SOFT);
    memtrack_unchecked_common_setup();
}
void memtrack_unchecked_oom_teardown()
{
    oomtest_disable();
    memtrack_unchecked_common_teardown();
}
/************************************************************************/
Suite* libdebug_memtrack_suite()
{
    Suite *s;
    TCase *tc;

    s = suite_create("LibDebug-MemTrack");
    tc = tcase_create("Uninitialized");
    tcase_add_checked_fixture(tc, memtrack_checked_uninitialized_setup, memtrack_checked_common_teardown);
    tcase_add_unchecked_fixture(tc, memtrack_unchecked_common_setup, memtrack_unchecked_common_teardown);
    suite_add_tcase(s, tc);
#ifndef NDEBUG
    tcase_add_test(tc, getallocatedRAM_first);
    tcase_add_test(tc, getallocatedblocks_first);
    tcase_add_test(tc, getblocksize_first);
    tcase_add_test(tc, getblocksize_null);
    tcase_add_test(tc, getblocksize_missing);
    tcase_add_test(tc, addblock_null_first);
    tcase_add_test(tc, delblock_null_first);
    tcase_add_test(tc, addblock_null);
    tcase_add_test(tc, addblock_duplicate);
    tcase_add_test(tc, addblock_empty);
    tcase_add_test(tc, addblock_longstrings);
    tcase_add_test(tc, delblock_null);
    tcase_add_test(tc, delblock_missing);
    tcase_add_test(tc, addblock1_delblock1);
    tcase_add_test(tc, addblock12_delblock12);
    tcase_add_test(tc, addblock12_delblock21);
    tcase_add_test(tc, addblock123_delblock123);
    tcase_add_test(tc, addblock123_delblock2);
    /* tcase_add_test(tc, adddelblock_oom); */
#endif

    tc = tcase_create("WithData");
    tcase_add_checked_fixture(tc, memtrack_checked_withdata_setup, memtrack_checked_common_teardown);
    tcase_add_unchecked_fixture(tc, memtrack_unchecked_common_setup, memtrack_unchecked_common_teardown);
    suite_add_tcase(s, tc);
#ifndef NDEBUG
    /* tcase_add_test(tc, getallocatedRAM_first); */
    /* tcase_add_test(tc, getallocatedblocks_first); */
    tcase_add_test(tc, getblocksize_first);
    tcase_add_test(tc, getblocksize_null);
    tcase_add_test(tc, getblocksize_missing);
    tcase_add_test(tc, addblock_null_first);
    tcase_add_test(tc, delblock_null_first);
    tcase_add_test(tc, addblock_null);
    tcase_add_test(tc, addblock_duplicate);
    tcase_add_test(tc, addblock_empty);
    tcase_add_test(tc, addblock_longstrings);
    tcase_add_test(tc, delblock_null);
    tcase_add_test(tc, delblock_missing);
    tcase_add_test(tc, addblock1_delblock1);
    tcase_add_test(tc, addblock12_delblock12);
    tcase_add_test(tc, addblock12_delblock21);
    tcase_add_test(tc, addblock123_delblock123);
    tcase_add_test(tc, addblock123_delblock2);
    /* tcase_add_test(tc, adddelblock_oom); */
#endif

    tc = tcase_create("Uninitialized-OOM");
    tcase_set_tags(tc,"oom");
    tcase_add_checked_fixture(tc, memtrack_checked_uninitialized_setup, memtrack_checked_common_teardown);
    tcase_add_unchecked_fixture(tc, memtrack_unchecked_oom_setup, memtrack_unchecked_oom_teardown);
    suite_add_tcase(s, tc);
#ifndef NDEBUG
    tcase_add_loop_test(tc, getallocatedRAM_first, 0, 10);
    tcase_add_loop_test(tc, getallocatedblocks_first, 0, 10);
    tcase_add_loop_test(tc, getblocksize_first, 0, 10);
    tcase_add_loop_test(tc, getblocksize_null, 0, 10);
    tcase_add_loop_test(tc, getblocksize_missing, 0, 10);
    tcase_add_loop_test(tc, addblock_null_first, 0, 10);
    tcase_add_loop_test(tc, delblock_null_first, 0, 10);
    tcase_add_loop_test(tc, addblock_null, 0, 10);
    tcase_add_loop_test(tc, addblock_duplicate, 0, 10);
    tcase_add_loop_test(tc, addblock_empty, 0, 10);
    tcase_add_loop_test(tc, addblock_longstrings, 0, 10);
    tcase_add_loop_test(tc, delblock_null, 0, 10);
    tcase_add_loop_test(tc, delblock_missing, 0, 10);
    tcase_add_loop_test(tc, addblock1_delblock1, 0, 10);
    tcase_add_loop_test(tc, addblock12_delblock12, 0, 10);
    tcase_add_loop_test(tc, addblock12_delblock21, 0, 10);
    tcase_add_loop_test(tc, addblock123_delblock123, 0, 10);
    tcase_add_loop_test(tc, addblock123_delblock2, 0, 10);
    tcase_add_loop_test(tc, adddelblock_oom, 0, 10);
#endif

    tc = tcase_create("WithData-OOM");
    tcase_set_tags(tc,"oom");
    tcase_add_checked_fixture(tc, memtrack_checked_withdata_setup, memtrack_checked_common_teardown);
    tcase_add_unchecked_fixture(tc, memtrack_unchecked_oom_setup, memtrack_unchecked_oom_teardown);
    suite_add_tcase(s, tc);
#ifndef NDEBUG
    /* tcase_add_loop_test(tc, getallocatedRAM_first, 0, 10); */
    /* tcase_add_loop_test(tc, getallocatedblocks_first, 0, 10); */
    tcase_add_loop_test(tc, getblocksize_first, 0, 10);
    tcase_add_loop_test(tc, getblocksize_null, 0, 10);
    tcase_add_loop_test(tc, getblocksize_missing, 0, 10);
    tcase_add_loop_test(tc, addblock_null_first, 0, 10);
    tcase_add_loop_test(tc, delblock_null_first, 0, 10);
    tcase_add_loop_test(tc, addblock_null, 0, 10);
    tcase_add_loop_test(tc, addblock_duplicate, 0, 10);
    tcase_add_loop_test(tc, addblock_empty, 0, 10);
    tcase_add_loop_test(tc, addblock_longstrings, 0, 10);
    tcase_add_loop_test(tc, delblock_null, 0, 10);
    tcase_add_loop_test(tc, delblock_missing, 0, 10);
    tcase_add_loop_test(tc, addblock1_delblock1, 0, 10);
    tcase_add_loop_test(tc, addblock12_delblock12, 0, 10);
    tcase_add_loop_test(tc, addblock12_delblock21, 0, 10);
    tcase_add_loop_test(tc, addblock123_delblock123, 0, 10);
    tcase_add_loop_test(tc, addblock123_delblock2, 0, 10);
    tcase_add_loop_test(tc, adddelblock_oom, 0, 10);
#endif

    return s;
}

